#include <stdio.h>
#include <math.h>
#include <limits.h>

#include <SDL.h>

void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
{
    if (x < 0 || y < 0 || x >= surface->w || y >= surface->h) return;

    int bpp = surface->format->BytesPerPixel;
    /*  p ϼԥؤɥ쥹 */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;
    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
}

//double target = 42164000.0;
double target = 21082000.0;
double EARTH = 6357000;

void clear(SDL_Surface* scr) {
    Uint32 color = SDL_MapRGB(scr->format, 255, 255, 255);
    SDL_FillRect(scr, 0, 0);
    for (int i = 0; i < 360; i++) {
        double r = i * M_PI / 180;
        int ix = cos(r) * 300 + 320;
        int iy = sin(r) * 300 + 320;
        putpixel(scr, ix, iy, color);
        ix = cos(r) * EARTH / target * 300 + 320;
        iy = sin(r) * EARTH / target * 300 + 320;
        putpixel(scr, ix, iy, color);

    }
}

int main() {
    SDL_Init(SDL_INIT_VIDEO);
    SDL_Surface* scr = SDL_SetVideoMode(640, 640, 32, SDL_SWSURFACE);
    Uint32 color = SDL_MapRGB(scr->format, 255, 255, 255);

    double best = 1e100;
    int bestTick = 0;

    int t = 0;
    int ok = 0;
    double x, y;
    while (scanf("%lf %lf %lf", &x, &y, &target) == 3) {
        if (t == 0) {
            clear(scr);
        }

        int ix = x / target * 300 + 320;
        int iy = y / target * 300 + 320;
        //printf("%f %f %d %d\n", x, y, ix, iy);
        putpixel(scr, ix, iy, color);
        t++;
        if (t % 100 == 0) {
            SDL_Flip(scr);
            //clear(scr);
        }

        double d = fabs(sqrt(x * x + y * y) - sqrt(target * target));
        if (d < best) {
            if (t % 10 == 0) {
                //printf("%d %lg %lf %lf\n", t, d, x, y);
            }
            best = d;
            bestTick = t;
        }

        if (d < 500) {
            //printf("OK: %d %d\n", t, ok);
            ok++;
        }
        else {
            ok = 0;
        }

/*
        if (t % 100 == 0) {
            printf("POS %d %lf %lf\n", t, x, y);
        }
*/
    }
    SDL_Flip(scr);

    printf("best: %d %lf\n", bestTick, best);

    SDL_Event ev;
    while (SDL_WaitEvent(&ev)) {
        if (ev.type == SDL_QUIT) {
            break;
        }
        else if (ev.type == SDL_KEYDOWN && ev.key.keysym.sym == SDLK_ESCAPE) {
            break;
        }
        SDL_Flip(scr);
    }

    SDL_Quit();
}
