#include <stdio.h>
#include <math.h>
#include <limits.h>
#include <algorithm>
#include <numeric>

using namespace std;

#include <SDL.h>

void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel) {
    if (x < 0 || y < 0 || x >= surface->w || y >= surface->h) return;

    int bpp = surface->format->BytesPerPixel;
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;
    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
}

//double target = 42164000.0;
//double target = 91082000.0;
double target = 1000000000000000.0;
double EARTH = 6357000;

void clear(SDL_Surface* scr) {
    Uint32 color = SDL_MapRGB(scr->format, 255, 255, 255);
    SDL_FillRect(scr, 0, 0);
    for (int i = 0; i < 360; i++) {
        double r = i * M_PI / 180;
        int ix = cos(r) * 300 + 320;
        int iy = sin(r) * 300 + 320;
        putpixel(scr, ix, iy, color);
        ix = cos(r) * EARTH / target * 300 + 320;
        iy = sin(r) * EARTH / target * 300 + 320;
        putpixel(scr, ix, iy, color);

    }
}

int main() {
    SDL_Init(SDL_INIT_VIDEO);
    SDL_Surface* scr = SDL_SetVideoMode(640, 640, 32, SDL_SWSURFACE);
    Uint32 color = SDL_MapRGB(scr->format, 255, 255, 255);
    Uint32 red = SDL_MapRGB(scr->format, 255, 0, 0);

    clear(scr);

    double best = 1e100;
    int bestTick = 0;

    int t = 0;
    int ok = 0;
    double x, y, ox, oy;
    while (scanf("%lf %lf", &x, &y) == 2) {
        SDL_Event ev;
        while (SDL_PollEvent(&ev)) {
            if (ev.type == SDL_QUIT) {
                break;
            }
            else if (ev.type == SDL_KEYDOWN && ev.key.keysym.sym == SDLK_ESCAPE) {
                break;
            }
        }

        int ix = x / target * 300 + 320;
        int iy = y / target * 300 + 320;
        //printf("%f %f %d %d\n", x, y, ix, iy);
        putpixel(scr, ix, iy, color);

        scanf("%lf %lf", &ox, &oy);
        double d = sqrt(ox * ox + oy * oy);
        ix = (ox = x - ox) / target * 300 + 320;
        iy = (oy = y - oy) / target * 300 + 320;
        putpixel(scr, ix, iy, red);

        if (target == 1000000000000000.0) {
            target = sqrt(ox*ox+oy*oy) * 3;
            // for 3003
            //target *= 20;
            printf("%lf\n", target);
        } else if (t % 100000 == 0) {
            target = max(target, sqrt(ox*ox+oy*oy) * 3);
        }

        t++;
        if (t % 1000 == 0) {
            SDL_Flip(scr);
            if (t % 1800 == 0) {
                clear(scr);
            }
        }

        //double d = fabs(sqrt(x * x + y * y) - sqrt(ox * ox + oy * oy));
        if (d < best && d < 100000) {
            /*
            if (t % 1000 == 0) {
                printf("%d %lg %lf %lf\n", t, d, x, y);
            }
            */
            best = d;
            bestTick = t;
        }

        if (d < 3000) {
            printf("OK: %d %d %lf\n", t, ok, d);
            ok++;
        }
        else {
            ok = 0;
        }

        if (t % 100000 == 0) {
            fprintf(stderr, "%d ticks\n", t);
        }
/*
        if (t % 100 == 0) {
            printf("POS %d %lf %lf\n", t, x, y);
        }
*/
    }
    SDL_Flip(scr);

    printf("target: %lf\n", target);
    printf("best: %d %lf\n", bestTick, best);

    SDL_Event ev;
    while (SDL_WaitEvent(&ev)) {
        if (ev.type == SDL_QUIT) {
            break;
        }
        else if (ev.type == SDL_KEYDOWN && ev.key.keysym.sym == SDLK_ESCAPE) {
            break;
        }
    }

    SDL_Quit();
}
