#include <glog/logging.h>

#include "actuator.h"

using namespace std;


int Actuator::step() {
    turn++;
    step_();
    int r = 0;
    while (!plans.empty() && turn == plans.front().time) {
        inaddr[r] = plans.front().addr;
        invalue[r] = plans.front().value;
        plans.pop_front();
        r++;
    }
    return r;
}

void Actuator::addPlan(int time, int addr, double value) {
    Plan plan;
    time += baseTurn;
    plan.time = time;
    plan.addr = addr;
    plan.value = value;
    LOG(INFO) << "Adding plan time=" << time
              << " addr=" << addr << " value=" << value;
    if (!plans.empty()) {
        CHECK_GE(time, plans.back().time);
    }
    plans.push_back(plan);
}

class NopActuator : public Actuator {
public:
    NopActuator() {}
    virtual void step_() {
    }
};

Actuator* createActuator1();
Actuator* createActuator2();
Actuator* createActuator3();
Actuator* createActuator4();

Actuator* createActuator(int problem, int* inaddr, double* invalue) {
    if (problem == -1) {
        return new NopActuator();
    }

    CHECK(problem > 0);
    CHECK(problem <= 4);
    Actuator* actuator = 0;
    if (problem == 1) {
        actuator = createActuator1();
    }
    else if (problem == 2) {
        actuator = createActuator3();
        // bye :)
        //actuator = createActuator2();
    }
    else if (problem == 3) {
        actuator = createActuator3();
    }
    else if (problem == 4) {
        actuator = createActuator4();
    }
    actuator->inaddr = inaddr;
    actuator->invalue = invalue;
    return actuator;
}
