#include <math.h>

#include <iomanip>
#include <vector>
#include <map>
#include <set>

#include <glog/logging.h>

#include "actuator.h"
#include "simulator.h"
#include "simulator4.h"
#include "util.h"

using namespace std;

static double freq(double d) {
    return 2 * M_PI * sqrt(d * d * d / MU);
}

class Actuator4 : public Actuator {
    typedef map<double, pair<double, double> > Ellipse;

public:
    Actuator4() {
        //nextHohmann = 3;
        for (int i = 0; i < 20; i++) nextHohmann[i] = 3;
        target = -1;
        act1 = 0;
        oxb.resize(20);
        oyb.resize(20);
        poxb.resize(20);
        poyb.resize(20);
        maxResource = 3500;
        killed = 0;
    }

    void calcMaxResource() {
        for (int i = 1; i < 12; i++) {
            Simulator4 sim(poxb[i], poyb[i], oxb[i], oyb[i],
                           mxa, mya, turn);
            double r = atan2(sim.vy, sim.vx);
            double r2 = atan2(sim.y, sim.x);
            double dr = fabs(normalize(r - r2));
            double res = 2500 * (dr / M_PI + 1) / 2;
            if (maxResource > res) {
                LOG(INFO) << "Reduced the max resource from "
                          << maxResource << " to " << res;
                maxResource = res;
            }
        }
        int rest = 12 - killed;
        double res = sdiv(out[1], rest);
        if (maxResource < res) {
            LOG(INFO) << "Few enemies adjustment: "
                      << maxResource << " to " << res;
            maxResource = res;
        }
    }

#if 0
    void adjustPlan(double x, double y, double ox, double oy) {
        if (turn % 100 != 0) return;
        LOG(INFO) << "Let's adjust our plan... turn=" << turn;

        Simulator4 osim(pox, poy, ox, oy, mxa, mya, turn);
        //Simulator osim(pox, poy, ox, oy);
        vector<double> oxa;
        vector<double> oya;
        osim.future(arrival - turn + 1000, &oxa, &oya);

        double best = 1e300;
        double bestDvx = 0, bestDvy = 0;
        for (int i = 0; i <= 0; i++) {
            double dvx = (i - 10) * 0.1;
            for (int j = 0; j <= 0; j++) {
                double dvy = (j - 10) * 0.1;

                Simulator4 sim(px, py, x, y, mxa, mya, turn);
                //sim.step(dvx, dvy);
                sim.step(0, 0);
                for (int k = 1; k < oxa.size(); k++) {
                    double d = dist(sim.x-oxa[k], sim.y-oya[k]);
                    if (best > d) {
                        best = d;
                        bestDvx = dvx;
                        bestDvy = dvy;
                    }
                    sim.step();
                }
            }
        }

        LOG(INFO) << "The best fix is " << best;

/*
        if (bestDvx != 0 || bestDvy != 0) {
            addPlan(turn, 2,-bestDvx);
            addPlan(turn, 3,-bestDvy);
            addPlan(turn+1, 2, 0);
            addPlan(turn+1, 3, 0);
        }
*/
    }
#endif

    bool calcPlan(double x, double y,
                  double ox, double oy, double pox, double poy,
                  int targ) {
        const int N = 200000;
/*
  if (N + turn + 6000 > 3000000) {
  N = 3000000 - turn - 6000;
  }
*/

        Simulator4 sim(px, py, x, y, mxa, mya, turn);
        //Simulator sim(px, py, x, y);
        //LOG(INFO) << x << ' ' << y << ' ' << px << ' ' << py;
        //LOG(INFO) << POS(sim.vx, sim.vy);
        vector<double> xa;
        vector<double> ya;
        sim.future(N, &xa, &ya);

/*
  for (int i = 0; i < 20000; i++) {
  LOG(INFO) << turn+i << ": " << POS(xa[i], ya[i]);
  }
*/

        Simulator4 osim(pox, poy, ox, oy, mxa, mya, turn);
        vector<double> oxa;
        vector<double> oya;
        vector<double> ovxa;
        vector<double> ovya;
        osim.future(N + 5000, &oxa, &oya, &ovxa, &ovya);

/*
  for (int i = 0; i < 20000; i++) {
  LOG(INFO) << turn+i << ": " << POS(oxa[i], oya[i]);
  }
*/
        //LOG(INFO) << POS(osim.vx, osim.vy);

        Ellipse ellipse;
        for (size_t i = 0; i < oxa.size(); i++) {
            double ox = oxa[i];
            double oy = oya[i];
            double r = atan2(oy, ox);
            ellipse.insert(make_pair(r, make_pair(ox, oy)));
        }

        int start = -1;
        double d, dd, t, dx, dy;
        // Find good position
        for (int i = 0; i < N; i++) {
            double x = xa[i];
            double y = ya[i];
            double r = atan2(y, x);
            double dr = r + M_PI;
            if (dr > M_PI) dr -= M_PI * 2;

            Ellipse::const_iterator found = ellipse.lower_bound(dr);
            Ellipse::const_iterator found2(found);
            ++found2;
            if (found2 == ellipse.end()) found2 = ellipse.begin();

            dx = found->second.first;
            dy = found->second.second;

            dd = sqrt(dx * dx + dy * dy);
            d = sqrt(x * x + y * y);

            double ddd = d + dd;

            t = M_PI * sqrt(ddd*ddd*ddd/8/MU);
            size_t tt = (size_t)(i + t);
            if (tt+1 >= oxa.size()) {
                continue;
            }

            double dt = i + t - tt;
            double oxe = oxa[tt] * (1 - dt) + oxa[tt+1] * dt;
            double oye = oya[tt] * (1 - dt) + oya[tt+1] * dt;
            double expected = dist(dx - oxe, dy - oye);

            if (expected < 10000) {
                LOG(INFO) << "Found";
                start = i;
                break;
            }
        }

        //CHECK_NE(start, -1);
        if (start == -1) {
            //nextHohmann[targ] = turn + N / 2;
            nextHohmann[targ] = turn + N / 5;
            //nextHohmann = turn + 10000;
            LOG(INFO) << "Not found :( next=" << nextHohmann[targ];
            return false;
        }

        LOG(INFO) << "Destination: " << POS(dx, dy);

        double dv0 = sqrt(G*M/d) * (sqrt(2*dd/(d+dd)) - 1);
        double dvt = sqrt(G*M/dd) * (1 - sqrt(2*d/(d+dd)));

        double r0 = dist(x, y);
        double dv0x = sdiv(-dv0 * ya[start], r0);
        double dv0y = sdiv(dv0 * xa[start], r0);
        double dvtx = sdiv(dvt * ya[start], r0);
        double dvty = sdiv(-dvt * xa[start], r0);

        clockwise = isClockwise(px, py, x, y);
        if (clockwise) {
            dv0x = -dv0x;
            dvtx = -dvtx;
        }

        if (false /*start > 200*/) {
            nextHohmann[targ] = turn + start - 100;
            LOG(INFO) << "Will think about the plan " << nextHohmann[targ];
        }
        else if (!decidePlan(xa, ya, oxa, oya, ovxa, ovya,
                             start, t, dv0x, dv0y, dvtx, dvty, targ)) {
            setPlans(start, t, dv0x, dv0y, dvtx, dvty, targ);
        }

        return !plans.empty();
    }

    virtual void step_() {
        double x = out[2];
        double y = out[3];
        double mx = x - out[0x64];
        double my = y - out[0x65];
        for (int i = 1; i < 13; i++) {
            oxb[i] = x - out[4+i*3];
            oyb[i] = y - out[5+i*3];
        }

        if (turn == 2) {
            mxa.push_back(0);
            mxa.push_back(pmx);
            //mxa.push_back(mx);
            mya.push_back(0);
            mya.push_back(pmy);
            //mya.push_back(my);
            LOG(INFO) << POS(mx,my) << POS(pmx,pmy);
            Simulator msim(pmx, pmy, mx, my);
            msim.future(3500000, &mxa, &mya);
        }

        if (turn % 1000 == 0 && killed) {
            calcMaxResource();
        }

#if 0
        if (act1) {
            if (!plans.empty()) {
                goto done_step;
            }
            act1->step();
            if (act1->arrivalTime() <= turn) {
                LOG(INFO) << "Finish radius adjustment"
                          << " turn=" << turn
                          << " arrive=" << act1->arrivalTime();
                delete act1;
                act1 = 0;
            }
            else {
                goto done_step;
            }
        }
#endif

        if (turn == arrival + 100) {
            LOG(INFO) << "What happened?";
            for (int i = 0; i < 20; i++) {
                nextHohmann[i] = turn;
            }
        }

        if (out[6+target*3] != 0 && target >= 0) {
            LOG(INFO) << "Finished target=" << target << " turn=" << turn;
            target = -1;
            targets.clear();
            arrival = -1;
            ++killed;
            for (int i = 0; i < 20; i++) {
                nextHohmann[i] = turn + 5;
            }

            calcMaxResource();
        }

        //if (target < 0) {
        if (target < 0 || (plans.empty() && turn > arrival + 100)) {
            int nextTarget = -1;
            double nearest = 1e300;
            double border = dist(x, y);
            for (int i = 1; i < 12; i++) {
                if (out[4+i*3+2] == 0) {
                    double ox = x - out[4+i*3];
                    double oy = y - out[4+i*3+1];
                    double od = dist(ox, oy);
                    if (nearest > od) {
                        nearest = od;
                        nextTarget = i;
                    }
                    if (border * 0.5 < od && od < border * 2) {
                        if (targets.insert(i).second) {
                            LOG(INFO) << "Target " << i
                                      << " was added because its near";
                        }
                    }
                }
            }
            if (target != nextTarget) {
                target = nextTarget;
                targets.insert(target);
                LOG(INFO) << "Next target: " << target
                          << " Score=" << out[0] << " fuel=" << out[1]
                          << " #targets=" << targets.size();
                google::FlushLogFiles(google::INFO);
                //nextHohmann = turn + 2;

                double od = dist(out[4+target*3], out[5+target*3]);
                double ot = freq(od);
                double d = dist(x, y);
                double t = freq(d);
                LOG(INFO) << "My freq=" << t << " target freq=" << ot;

#if 0
                if (ot * 0.4 < t) {
                    LOG(INFO) << "Out frequencies are too close";
                    act1 = createActuator(1, inaddr, invalue);
                    act1->out = out;
                    act1->baseTurn = turn;
                    double tmp = ot * 0.2 / (4 * M_PI);
                    double goal = cbrt(tmp * tmp * MU);
                    act1->setGoal(goal);
                    //act1->setGoal(2 * d - od);
                    goto done_step;
                }
#endif
            }
        }

        double ox = x - out[4+target*3];
        double oy = y - out[5+target*3];

#if 0
        if (departure + 10 < turn && turn < arrival) {
            adjustPlan(x, y, ox, oy);
        }
#endif

        if (turn == arrival) {
            LOG(INFO) << "The expected arrival time: "
                      << POS(x, y) << " vs " << POS(ox, oy)
                      << " " << dist(x-ox, y-oy);
        }

        for (set<int>::const_iterator iter = targets.begin();
             iter != targets.end() && plans.empty();
             ++iter) {
            int t = *iter;
            if (turn >= nextHohmann[t]) {
                LOG(INFO) << "Let's plan the trip turn=" << turn
                          << " for " << t;

/*
            calcPlan(x, y,
                     oxb[target], oyb[target], poxb[target], poyb[target]);
*/

                if (out[6+t*3] != 0) {
                    LOG(ERROR) << "Target was already dead? " << t;
                }
                else {
                    if (calcPlan(x, y, oxb[t], oyb[t], poxb[t], poyb[t], t)) {
                        if (target != t) {
                            LOG(INFO) << "Target changed: " << target
                                      << " => " << t;
                        }
                        target = t;
                        break;
                    }
                }
            }
        }

        //done_step:
        px = x;
        py = y;
        pmx = mx;
        pmy = my;
        for (int i = 1; i < 13; i++) {
            poxb[i] = oxb[i];
            poyb[i] = oyb[i];
        }
    }

    void setPlans(int start, double t,
                  double dv0x, double dv0y, double dvtx, double dvty,
                  int targ) {
        addPlan(turn+start, 2, dv0x);
        addPlan(turn+start, 3, dv0y);
        addPlan(turn+start+1, 2, 0);
        addPlan(turn+start+1, 3, 0);

        t += turn + start;
        int it = (int)t;
        double dt = t - it;
        addPlan(it, 2, dvtx * (1 - dt));
        addPlan(it, 3, dvty * (1 - dt));
        addPlan(it + 1, 2, dvtx * dt);
        addPlan(it + 1, 3, dvty * dt);
        addPlan(it + 2, 2, 0);
        addPlan(it + 2, 3, 0);

        nextHohmann[targ] = it + 2000;

        departure = turn + start;
        arrival = t;
        LOG(INFO) << "We start at " << (int)(turn+start)
                  << " with " << POS(dv0x, dv0y)
                  << " and will arrive at " << (int)(t)
                  << " with " << POS(dvtx, dvty)
                  << " next trip will be " << nextHohmann[targ];
    }

    bool willSucceed(double x0, double y0, double x1, double y1,
                     const vector<double>& oxa, const vector<double>& oya,
                     int start, double t,
                     double dv0x, double dv0y, double dvtx, double dvty,
                     double* best) {
        Simulator4 sim(x0, y0, x1, y1, mxa, mya, turn+start);
        //Simulator sim(x0, y0, x1, y1);
        //LOG(INFO) << POS(sim.x, sim.y);
        sim.step(dv0x, dv0y);
        for (int i = 1; i < t-1; i++) {
            //LOG(INFO) << "turn=" << turn+start+i << POS(sim.x, sim.y);
            sim.step();
        }
        double dt = t - (int)t;
        //LOG(INFO) << "turn=" << turn+start+t << POS(sim.x, sim.y);
        //LOG(INFO) << POS(dvtx * (1 - dt), dvty * (1 - dt));
        sim.step(dvtx * (1 - dt), dvty * (1 - dt));
        //LOG(INFO) << "turn=" << turn+start+t+1 << POS(sim.x, sim.y);
        //LOG(INFO) << POS(dvtx * dt, dvty * dt);
        sim.step(dvtx * dt, dvty * dt);
        int ok = 0;
        int ot = start + t + 2;
        *best = 1e300;
        for (int i = 0; i < 4000; i++) {
/*
            LOG(INFO) << "turn=" << turn+start+t+i+2
                      << POS(sim.x, sim.y) << " vs " << POS(oxa[ot+i], oya[ot+i]);
*/

            double d = dist(sim.x - oxa[ot + i], sim.y - oya[ot + i]);
            if (*best > d) {
                *best = d;
            }
            if (d < 1000) {
                ok++;
                //fprintf(stderr, "%d %f\n", ok, d);
                if (ok > 900) return true;
            }
            else {
                ok = 0;
            }
            sim.step();
        }
        return false;
    }

    double adjustDv(const vector<double>& xa, const vector<double>& ya,
                    const vector<double>& oxa, const vector<double>& oya,
                    double px, double py,
                    int start, double t,
                    double dv0x, double dv0y,
                    int cnt, double scale,
                    double* bestDv0xo, double* bestDv0yo) {
        double best;
        double bestX = 1e300;
        double bestDv0x = 0;
        for (int i = 0; i <= cnt; i++) {
            double ndv0x = dv0x + (i / (double)cnt - 0.5) * scale;
            willSucceed(px, py, xa[start], ya[start], oxa, oya,
                        start, t, ndv0x, dv0y, 0, 0, &best);
            if (bestX > best) {
                bestX = best;
                bestDv0x = ndv0x;
            }
        }
        LOG(INFO) << "bestX=" << bestX << " bestDv0x=" << bestDv0x;

        double bestY = 1e300;
        double bestDv0y = 0;
        for (int i = 0; i <= cnt; i++) {
            double ndv0y = dv0y + (i / (double)cnt - 0.5) * scale;
            willSucceed(px, py, xa[start], ya[start], oxa, oya,
                        start, t, dv0x, ndv0y, 0, 0, &best);
            if (bestY > best) {
                bestY = best;
                bestDv0y = ndv0y;
            }
        }
        LOG(INFO) << "bestY=" << bestY << " bestDv0y=" << bestDv0y;

        willSucceed(px, py, xa[start], ya[start], oxa, oya,
                    start, t, bestDv0x, bestDv0y, 0, 0, &best);
        LOG(INFO) << "best=" << best
                  << " bestX=" << bestX << " bestY=" << bestY;
        if (best > bestX && bestY > bestX) {
            bestDv0y = dv0y;
            best = bestX;
        }
        else if (best > bestY && bestX > bestY) {
            bestDv0x = dv0x;
            best = bestY;
        }

        *bestDv0xo = bestDv0x;
        *bestDv0yo = bestDv0y;
        return best;
    }

    bool decidePlan(const vector<double>& xa, const vector<double>& ya,
                    const vector<double>& oxa, const vector<double>& oya,
                    const vector<double>& ovxa, const vector<double>& ovya,
                    int start, double t,
                    double dv0x, double dv0y, double dvtx, double dvty,
                    int targ) {
        double px, py;
        if (start > 0) {
            px = xa[start-1];
            py = ya[start-1];
        }
        else {
            px = this->px;
            py = this->py;
        }

        double best;
        if (willSucceed(px, py, xa[start], ya[start], oxa, oya,
                        start, t, dv0x, dv0y, dvtx, dvty, &best)) {
            return false;
        }
        LOG(INFO) << "best=" << best << " dv0x=" << dv0x << " dv0y=" << dv0y;

        double bestDv0x = dv0x, bestDv0y = dv0y;
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 10000, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 5000, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 1000, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 500, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 200, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 100, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 50, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 20, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 10, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 5, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 2, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 1, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 0.5, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 0.2, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 0.1, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 0.05, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 0.02, &bestDv0x, &bestDv0y);
        best = adjustDv(xa, ya, oxa, oya, px, py, start, t, bestDv0x, bestDv0y,
                        10, 0.01, &bestDv0x, &bestDv0y);

        if (best >= 1000) {
            nextHohmann[targ] = turn + start + 1000;
            LOG(INFO) << "Our plan was not good. Wait for the next chance: "
                      << nextHohmann[targ];
            return true;
        }

        double resource = dist(bestDv0x, bestDv0y);
        int firstBoost = turn + start;

        Simulator4 sim(px, py, xa[start], ya[start], mxa, mya, turn+start);
        //Simulator sim(px, py, xa[start], ya[start]);
        sim.step(bestDv0x, bestDv0y);
        int i;
        for (i = 1;; i++) {
            CHECK_LT(start+i, oxa.size());

            double d = dist(sim.x - oxa[start+i], sim.y - oya[start+i]);
            if (d == best) break;

            sim.save();
            sim.step();
        }

        sim.restore();

        double nvx = ovxa[start+i];
        double nvy = ovya[start+i];
        if (clockwise != isClockwise(oxa[0], oya[0], oxa[1], oya[1])) {
            nvx = -nvx;
            nvy = -nvy;
        }

        double cv = sqrt(MU / dist(sim.x, sim.y));
        double cr = atan2(sim.y, sim.x) + M_PI / 2;
        double cvx = cos(cr) * cv;
        double cvy = sin(cr) * cv;
        if (!clockwise) {
            cvx = -cvx;
            cvy = -cvy;
        }
        nvx = cvx;
        nvy = cvy;

#if 1
        double dvx = nvx - sim.vx;
        double dvy = nvy - sim.vy;

        resource += dist(dvx, dvy);
        if (resource > maxResource) {
            LOG(INFO) << "Too expensive";
            nextHohmann[targ] = turn + start + 1000;
            return true;
        }
        else if (resource > out[1] - 3) {
            LOG(INFO) << "Not enough resource";
            nextHohmann[targ] = turn + start + 1000;
            return true;
        }

        addPlan(firstBoost, 2, bestDv0x);
        addPlan(firstBoost, 3, bestDv0y);
        addPlan(firstBoost+1, 2, 0);
        addPlan(firstBoost+1, 3, 0);

        addPlan(turn+start+i-1, 2, -dvx);
        addPlan(turn+start+i-1, 3, -dvy);

/*
        for (int j = 0; j < 5000; j++) {
            i++;
            sim.step(-dvx, -dvy);
            dvx = ovxa[start+i] - sim.vx;
            dvy = ovya[start+i] - sim.vy;
            LOG(INFO) << POS(dvx, dvy);
            addPlan(turn+start+i-1, 2, -dvx);
            addPlan(turn+start+i-1, 3, -dvy);
        }
*/

        addPlan(turn+start+i, 2, 0);
        addPlan(turn+start+i, 3, 0);
#endif

        departure = turn + start;
        arrival = turn+start+i;
        LOG(INFO) << "We start at " << (int)(turn+start)
                  << " with " << POS(dv0x, dv0y)
                  << " and will arrive at " << (int)(arrival)
                  << " with " << POS(dvtx, dvty)
                  << " we should be in " << POS(sim.x, sim.y)
                  << " target should be in " << POS(oxa[start+i], oya[start+i])
                  << " next trip will be " << nextHohmann[targ];

        //nextHohmann = turn+start+i+5000;

        return true;
    }

    vector<double> mxa, mya;
    vector<double> oxb, oyb;
    vector<double> poxb, poyb;
    int target;
    double px, py;
    double pmx, pmy;
    int nextHohmann[20];
    bool clockwise;
    int departure, arrival;
    Actuator* act1;
    set<int> targets;
    double maxResource;
    int killed;
};

Actuator* createActuator4() {
    return new Actuator4();
}
