#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <vector>
#include <map>

using namespace std;

double dis(double x, double y) {
    double z = x - y;
    return z * z;
}

int main(int argc, char* argv[]) {
    if (argc <= 1) {
        fprintf(stderr, "argv!\n");
        exit(1);
    }

    double G = 6.67428e-11;
    double M = 6.0e24;

    fprintf(stderr, "loading... :)\n");
    vector<double> xs;
    vector<double> ys;
    vector<double> oxs;
    vector<double> oys;
    double x, y, ox, oy;
    while (scanf("%lf%lf%lf%lf",&x,&y,&ox,&oy) == 4) {
        xs.push_back(x);
        ys.push_back(y);
        ox = x - ox;
        oy = y - oy;
        oxs.push_back(ox);
        oys.push_back(oy);
    }

    fprintf(stderr, "map generation... :)\n");
    typedef map<double, pair<double, double> > Ellipse;
    Ellipse ellipse;
    double fox = 0, foy = 0;
    int state = 0;
    double pd = 0;
    for (size_t i = 0; i < oxs.size(); i++) {
        double ox = oxs[i];
        double oy = oys[i];
        double d = 0;
        if (fox == 0.0) {
            fox = ox;
            foy = oy;
        }
        else {
            d = sqrt(dis(fox, ox) + dis(foy, oy));
            if (state == 0 && d < pd) {
                state = 1;
            }
            else if (state == 1 && d > pd) {
                //break;
            }
            pd = d;
        }

        double r = atan2(oy, ox);
        //printf("%lf %lf %lf %lf\n", ox, oy, d, r);
        ellipse.insert(make_pair(r, make_pair(ox, oy)));
    }

    fprintf(stderr, "searching... :)\n");
    double start = 0, st = 0;
    double t = 0, d = 0, dd = 0;
    int ok = 0;
    double prevExpected = 10000;
#define SLICE 100
    for (size_t i = 0; i < xs.size() * SLICE - SLICE; i++, st += 1.0/SLICE) {
        double dst = st - (size_t)st;
        x = xs[i/SLICE] * (1 - dst) + xs[i/SLICE+1] * dst;
        y = ys[i/SLICE] * (1 - dst) + ys[i/SLICE+1] * dst;
        double r = atan2(y, x);
        double dr = r + M_PI;
        if (dr > M_PI) dr -= M_PI * 2;

        Ellipse::const_iterator found = ellipse.lower_bound(dr);
        Ellipse::const_iterator found2(found);
        ++found2;
        if (found2 == ellipse.end()) found2 = ellipse.begin();

        //fprintf(stderr, "%lf %lf %lf\n", found->first, dr, found2->first);
        // naive
#if 0
        double ddr1 = fabs(found->first - dr);
        double ddr2 = fabs(found2->first - dr);
        double ddrt = ddr1 + ddr2;
        ddr1 /= ddrt;
        ddr2 /= ddrt;
        double dx = (found->second.first * ddr2 + found2->second.first * ddr1);
        double dy = (found->second.second * ddr2 + found2->second.second * ddr1);
#endif
        double dx = found->second.first;
        double dy = found->second.second;

        dd = sqrt(dx * dx + dy * dy);
        d = sqrt(x * x + y * y);

        double ddd = d + dd;
        t = M_PI * sqrt(ddd*ddd*ddd/8/G/M);
        size_t tt = (size_t)(st + t);
        if (tt+1 >= oxs.size()) {
            //fprintf(stderr, "not found\n");
            //exit(1);
            continue;
        }

        double dt = st + t - tt;
        double oxe = oxs[tt] * (1 - dt) + oxs[tt+1] * dt;
        double oye = oys[tt] * (1 - dt) + oys[tt+1] * dt;
        double expected = sqrt(dis(dx, oxe) + dis(dy, oye));

        //fprintf(stderr, "%lf\n", expected);
#if 0
        if (expected < 1000)
#else  // for 3003
        if (expected < 10 && expected < prevExpected)
#endif
        {
            fprintf(stderr, "found\n");
            ++ok;
            start = st;
            prevExpected = expected;
        }
        //if (ok && ++ok > 1000) break;
        if (ok) break;
    }

    if (!ok) {
        fprintf(stderr, "not found\n");
        exit(1);
    }

    double dv0 = sqrt(G*M/d) * (sqrt(2*dd/(d+dd)) - 1);
    double dvt = sqrt(G*M/dd) * (1 - sqrt(2*d/(d+dd)));

    double dv0x = d == 0.0 ? 0.0 : - dv0 * y / d;
    double dv0y = d == 0.0 ? 0.0 : dv0 * x / d;
    double dvtx = d == 0.0 ? 0.0 : dvt * y / d;
    double dvty = d == 0.0 ? 0.0 : - dvt * x / d;

    puts(argv[1]);

    puts("0");
    puts("1");
    printf("16000 %s\n", argv[1]);

    size_t ist = (size_t)start;
    double dst = start - (size_t)start;
    printf("%lu\n", ist);
    puts("2");
    printf("2 %.19lf\n", dv0x * (1-dst));
    printf("3 %.19lf\n", dv0y * (1-dst));

    printf("%lu\n", ist + 1);
    puts("2");
    printf("2 %.19lf\n", dv0x * dst);
    printf("3 %.19lf\n", dv0y * dst);

    printf("%lu\n", ist + 2);
    puts("2");
    printf("2 %.19lf\n", 0.0);
    printf("3 %.19lf\n", 0.0);

#if 0
    double dt = t - (size_t)t;

    printf("%lu\n", start + (size_t)t + 1);
    puts("2");
    printf("2 %.19lf\n", dvtx * (1 - dt));
    printf("3 %.19lf\n", dvty * (1 - dt));

    printf("%lu\n", start + (size_t)t + 2);
    puts("2");
    printf("2 %.19lf\n", dvtx * dt);
    printf("3 %.19lf\n", dvty * dt);

    printf("%lu\n", start + (size_t)t + 3);
    puts("2");
    printf("2 %.19lf\n", 0.0);
    printf("3 %.19lf\n", 0.0);
#endif

    puts("100000");
    puts("0");

    char buf[256];
    sprintf(buf, "tmp%s.rb", argv[1]);
    FILE* fp = fopen(buf, "wb");
    fprintf(fp, "%d\n", (size_t)start + (size_t)t);
    fclose(fp);
}
