#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <stdint.h>
#include <math.h>

#include <vector>

#include <gflags/gflags.h>
#include <glog/logging.h>

typedef unsigned int uint;

using namespace std;
using namespace google;

DEFINE_int32(max_turn, 500000, "");
DEFINE_int32(machine, 0, "");
DEFINE_int32(verbose, 0, "");
DEFINE_int32(save, -1, "");
DEFINE_int32(finish_scored, 0, "");

enum {
    ADD, SUB, MUL, DIV, OUT, PHI, NOP, CMPZ, SQRT, COPY, INPUT,
};
static const char* OP_STR[] = {
    "ADD", "SUB", "MUL", "DIV", "OUT", "PHI",
    "NOP", "CMPZ", "SQRT", "COPY", "INPUT",
};

struct Op {
    int op;
    int r;
    union {
        int i;
        int r2;
    };
};

struct In {
    uint addr;
    double value;
};

struct Frame {
    int time;
    vector<In> inputs;
};

int main(int argc, char* argv[]) {
    ParseCommandLineFlags(&argc, &argv, true);
    InitGoogleLogging(argv[0]);

    CHECK_GT(argc, 2);

    FILE* fp = fopen(argv[1], "rb");
    fseek(fp, 0, SEEK_END);
    long len = ftell(fp);
    fseek(fp, 0, SEEK_SET);
    uint* buf = (uint*)malloc(len);
    fread(buf, 1, len, fp);
    fclose(fp);

    vector<Frame*> frames;
    fp = fopen(argv[2], "rb");
    fseek(fp, 12, SEEK_SET);
    int tf;
    if (FLAGS_verbose) puts("OSF");
    while (fread(&tf, 4, 1, fp) == 1) {
        Frame* f = new Frame;
        f->time = tf;
        frames.push_back(f);
        int n;
        CHECK_EQ(fread(&n, 4, 1, fp), 1);
        for (int i = 0; i < n; i++) {
            int a;
            double d;
            CHECK_EQ(fread(&a, 4, 1, fp), 1);
            CHECK_EQ(fread(&d, 8, 1, fp), 1);
            In input;
            input.addr = a;
            input.value = d;
            f->inputs.push_back(input);
        }
    }
    fclose(fp);

    if (FLAGS_verbose) puts("OPS");
    Op* ops = (Op*)malloc(len);
    len /= 4;
    for (int i = 0; i < len; i += 3) {
        uint inst = buf[i % 2 == 0 ? i + 2 : i];
        uint op = inst >> 28;
        uint r1, r2;
        if (op) {
            op = op - 1;
            DCHECK_LT(op, NOP);
            r1 = (inst >> 14) & 0x3fff;
            r2 = inst & 0x3fff;
        }
        else {
            op = NOP + (inst >> 24);
            DCHECK_LE(op, INPUT);
            r1 = inst & 0x3fff;
            r2 = (inst >> 21) & 7;
#if 0
            if (r2) {
                CHECK_EQ(r2, 4);
                r2 = 1;
            }
#endif
        }
        int a = i / 3;
        ops[a].op = op;
        ops[a].r = r1;
        ops[a].r2 = r2;
        if (FLAGS_verbose) printf("%u: %s %u %u %x\n", i/3, OP_STR[op], r1, r2, inst);
    }

    if (FLAGS_verbose) puts("MEMORY");
    double mem[16384];
    for (int i = 0; i < len; i += 3) {
        uint* p = buf + (i % 2 == 0 ? i : i + 1);
        mem[i / 3] = *(double*)p;
        if (FLAGS_verbose) printf("%d: %f\n", i / 3, mem[i / 3]);
    }

    double out[16384];
    double port[16384];
    for (int i = 0; i < 16384; i++) {
        out[i] = port[i] = 0;
    }

    if (FLAGS_verbose) puts("BOOT");

#if 0
    port[0x3e80] = 1001;
    //port[2] = -10000;
    port[3] = -2466.48601221270837413612753152847;
    //port[3] = -2466;
    //port[0x3e80] = 2001;
#endif

    len /= 3;

    bool status = false;
    bool scored = false;
    int t;
    int inputIndex = 0;
    for (t = 0; t < FLAGS_max_turn /* && out[0] == 0*/ ; t++) {
        const Frame* frame = frames[inputIndex];
        if (frame->time == t) {
            int n = frame->inputs.size();
            if (n == 0) break;
            for (int j = 0; j < n; j++) {
                In input = frame->inputs[j];
                port[input.addr] = input.value;
                if (FLAGS_verbose) printf("setting port: %d %lf\n", input.addr, input.value);
            }
            inputIndex++;
        }

        if (!FLAGS_machine) printf("TURN: %d\n", t);
        for (int i = 0; i < len; i++) {
            //printf("%d %s %u %u\n", i, OP_STR[ops[i].op], ops[i].r, ops[i].r2);
            switch (ops[i].op) {
            case ADD:
                mem[i] = mem[ops[i].r] + mem[ops[i].r2];
                break;
            case SUB:
                mem[i] = mem[ops[i].r] - mem[ops[i].r2];
                //mem[i] = mem[ops[i].r2] - mem[ops[i].r];
                //printf("SUB: %f-%f=%f\n", mem[ops[i].r], mem[ops[i].r2], mem[i]);
                break;
            case MUL:
                mem[i] = mem[ops[i].r] * mem[ops[i].r2];
                break;
            case DIV: {
                double d = mem[ops[i].r2];
                mem[i] = d != 0 ? mem[ops[i].r] / d : 0.0;
                break;
            }
            case OUT:
                out[ops[i].r] = mem[ops[i].r2];
                if (!FLAGS_machine) {
                    printf("OUTPUT: %d: %f\n", ops[i].r, mem[ops[i].r2]);
                }
                break;
            case PHI:
                //mem[i] = status ? mem[ops[i].r] : mem[ops[i].r2];
                mem[i] = status ? mem[ops[i].r] : mem[ops[i].r2];
                break;
            case NOP:
                break;
            case CMPZ:
                switch (ops[i].i) {
                case 0: /* LTZ */
                    //status = mem[ops[i].r] < 1e100;
                    status = mem[ops[i].r] < 0.0;
                    break;
                case 1: /* LEZ */
                    status = mem[ops[i].r] <= 0.0;
                    break;
                case 2: /* EQZ */
                    status = mem[ops[i].r] == 0.0;
                    break;
                case 3: /* GEZ */
                    status = mem[ops[i].r] >= 0.0;
                    break;
                case 4: /* GTZ */
                    //status = mem[ops[i].r] > -1e100;
                    status = mem[ops[i].r] > -0.0;
                    //status = fabs(mem[ops[i].r]) < 1e100;
                    break;
                default:
                    LOG(FATAL) << "Shouldn't happen";
                }
                //printf("CMPZ: %d\n", status);
                break;
            case SQRT:
                CHECK_GE(mem[ops[i].r], 0);
                mem[i] = sqrt(mem[ops[i].r]);
                break;
            case COPY:
                mem[i] = mem[ops[i].r];
                break;
            case INPUT:
                mem[i] = port[ops[i].r];
                //printf("INPUT: %d: %f\n", ops[i].r, mem[i]);
                break;
            default:
                LOG(FATAL) << "Shouldn't happen";
            }
            //printf("%f\n", mem[i]);
        }

        if (FLAGS_machine) {
            printf("%f %f %f %f\n", out[2], out[3], out[4], out[5]);
        }

        if (out[0] && !scored) {
            if (!FLAGS_machine) {
                printf("SCORE! %f\n", out[0]);
            }
            if (FLAGS_finish_scored) {
                if (!FLAGS_machine) {
                    printf("%d\n", t+1);
                }
                fprintf(stderr, "finished %d %f %f\n", t, out[0], out[1]);
                return 0;
            }
            scored = true;
        }
    }
    fprintf(stderr, "finished %d %f %f\n", t, out[0], out[1]);
}
