#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <stdint.h>
#include <math.h>
#include <assert.h>

#include <sys/stat.h>
#include <sys/types.h>

#include <vector>

#include <glog/logging.h>
#include <gflags/gflags.h>

#include "actuator.h"
#include "osf.h"
#include "util.h"
#include "vis.h"
#include "vm.h"

using namespace std;

DEFINE_int32(max_turn, 500000, "");
DEFINE_bool(out, false, "");
DEFINE_bool(pretty, false, "");
DEFINE_bool(vis, false, "");
DEFINE_bool(nop, false, "");

static int NUM_OUTPUTS[] = {
    -1, 5, 6, 6, 4 + 3 + 3*11
};

int main(int argc, char* argv[]) {
    google::ParseCommandLineFlags(&argc, &argv, true);
    google::InitGoogleLogging(argv[0]);
    google::InstallFailureSignalHandler();

    CHECK_GT(argc, 2);

    int scenario = atoi(argv[2]);
    {
        char buf[256];
        sprintf(buf, "%d", scenario);
        FLAGS_log_dir = buf;

        mkdir(FLAGS_log_dir.c_str(), 0755);

        if (scenario >= 3000) {
            FLAGS_max_turn = 3000000;
        }
    }

    VM* vm = createVM(argv[1]);
    double* out = vm->getOutput();
    int inaddr[99];
    double invalue[99];

    int problem = scenario / 1000;
    assert(problem > 0);
    assert(problem <= 4);
    Actuator* actuator = createActuator(FLAGS_nop ? -1 : problem,
                                        inaddr, invalue);
    actuator->out = out;

    OSF osf(scenario);

    Vis* vis = 0;
    if (FLAGS_vis) {
        vis = createVis(problem);
        vis->out = out;
    }

    vm->setInput(0x3e80, scenario);

    for (int t = 0; t < FLAGS_max_turn; t++) {
        vm->step();

        if (out[0]) {
            osf.out(t + 2);
            osf.out(0);

            if (out[0] > 0) {
                fprintf(stderr, "Success: %lf %lf %d\n", out[0], out[1], t);
                LOG(INFO) << "Succeeded";
            }
            else {
                fprintf(stderr, "FAILED: %lf %lf %d\n", out[0], out[1], t);
                LOG(INFO) << "FAILED";
            }
            break;
        }

        if (FLAGS_pretty || (!FLAGS_out && t % 100000 == 99999)) {
            if (problem == 4) {
                printf("%d-%d (%lf, %lf)\n",
                       t, t+1, out[2], out[3]);
                printf("MOON (%lf, %lf) %lf\n",
                       out[2]-out[0x64], out[3]-out[0x65],
                       dist(out[0x64], out[0x65]));
                printf("FUEL (%lf, %lf) %lf\n",
                       out[2]-out[4], out[3]-out[5],
                       dist(out[4], out[5]));
                for (int i = 0; i < 11; i++) {
                    printf(" T%02d (%lf, %lf) %lf\n",
                           i, out[2]-out[7+i*3], out[3]-out[8+i*3],
                           dist(out[7+i*3], out[8+i*3]));
                }
            }
            else {
                printf("%d-%d (%lf, %lf) (%lf, %lf) %lf\n",
                       t, t+1, out[2], out[3], out[2]-out[4], out[3]-out[5],
                       dist(out[4], out[5]));
            }
        }
        else if (FLAGS_out) {
            for (int i = 0; i < NUM_OUTPUTS[problem]; i++) {
                printf("%lf ", out[i]);
            }
            if (problem == 4) {
                printf("%lf %lf\n", out[0x64], out[0x65]);
            }
            else {
                printf("%d-%d\n", t, t+1);
            }
        }

        int inputCnt = actuator->step();
        if (inputCnt) {
            //printf("inputs %f %f\n", invalue[0], invalue[1]);
            osf.out(t + 1);
            osf.out(inputCnt);
            for (int i = 0; i < inputCnt; i++) {
                LOG(INFO) << "Input: " << t << " "
                          << inaddr[i] << ": " << invalue[i];

                if (FLAGS_pretty) {
                    printf("Input: %d: %lf\n", inaddr[i], invalue[i]);
                }

                osf.outv(inaddr[i], invalue[i]);
                vm->setInput(inaddr[i], invalue[i]);
            }
        }

        if (t % 10000 == 0) {
            google::FlushLogFiles(google::INFO);
        }
    }

    LOG(INFO) << "Finished: " << out[0] << ' ' << out[1];

    delete actuator;
    delete vm;
}
