#include "simulator.h"
#include "util.h"

#include <glog/logging.h>

static void gravity(double x, double y, double* g0x, double* g0y) {
    double d20 = dist2(x, y);
    double g0 = MU / d20;
    double d0 = sqrt(d20);
    *g0x = - g0 * x / d0;
    *g0y = - g0 * y / d0;
}

Simulator::Simulator(double x0, double y0, double x1, double y1,
                     double dvx0, double dvy0) {
    this->x = this->x1 = x1;
    this->y = this->y1 = y1;

    double g0x, g0y;
    gravity(x0, y0, &g0x, &g0y);

    double dx = x1 - x0;
    double dy = y1 - y0;
    double vx0 = dx - 0.5 * (g0x + dvx0);
    double vy0 = dy - 0.5 * (g0y + dvy0);

    double g1x, g1y;
    gravity(x1, y1, &g1x, &g1y);
    vx1 = vx = vx0 + dvx0 + 0.5 * (g1x + g0x);
    vy1 = vy = vy0 + dvy0 + 0.5 * (g1y + g0y);

    //LOG(INFO) << "vx=" << vx << " vy=" << vy;
}

void Simulator::step(double dvx, double dvy) {
    dvx = -dvx;
    dvy = -dvy;

    double g0x, g0y;
    gravity(x, y, &g0x, &g0y);

    x += vx + 0.5 * (g0x + dvx);
    y += vy + 0.5 * (g0y + dvy);

    double g1x, g1y;
    gravity(x, y, &g1x, &g1y);

    vx += dvx + 0.5 * (g0x + g1x);
    vy += dvy + 0.5 * (g0y + g1y);
}

void Simulator::future(int n, vector<double>* xa, vector<double>* ya) {
    xa->reserve(n);
    ya->reserve(n);
    xa->push_back(x);
    ya->push_back(y);
    for (int i = 0; i < n; i++) {
        step();
        xa->push_back(x);
        ya->push_back(y);
    }
}

void Simulator::future(int n, vector<double>* xa, vector<double>* ya,
                       vector<double>* vxa, vector<double>* vya) {
    xa->reserve(n);
    ya->reserve(n);
    vxa->reserve(n);
    vya->reserve(n);
    xa->push_back(x);
    ya->push_back(y);
    vxa->push_back(vx);
    vya->push_back(vy);
    for (int i = 0; i < n; i++) {
        step();
        xa->push_back(x);
        ya->push_back(y);
        vxa->push_back(vx);
        vya->push_back(vy);
    }
}

void Simulator::save() {
    x1 = x;
    y1 = y;
    vx1 = vx;
    vy1 = vy;
}

void Simulator::restore() {
    x = x1;
    y = y1;
    vx = vx1;
    vy = vy1;
}
