#include "simulator4.h"
#include "util.h"

#include <glog/logging.h>

static void gravity(double x, double y, double* g0x, double* g0y) {
    double d20 = dist2(x, y);
    double g0 = MU / d20;
    double d0 = sqrt(d20);
    *g0x = - g0 * x / d0;
    *g0y = - g0 * y / d0;
}

static void gravity2(double x, double y, double mx, double my,
                     double* g0x, double* g0y) {
    gravity(x, y, g0x, g0y);
    double dx = mx - x;
    double dy = my - y;
    double d20 = dist2(dx, dy);
    double g0 = MUm / d20;
    double d0 = sqrt(d20);
    *g0x += g0 * dx / d0;
    *g0y += g0 * dy / d0;
}

Simulator4::Simulator4(double x0, double y0, double x1, double y1,
                       const vector<double>& mxa0, const vector<double>& mya0,
                       int t,
                       double dvx0, double dvy0)
    : mxa(mxa0), mya(mya0)
{
    turn = t;
    this->x = this->x1 = x1;
    this->y = this->y1 = y1;

    double g0x, g0y;
    gravity2(x0, y0, mxa[turn-1], mya[turn-1], &g0x, &g0y);

    double dx = x1 - x0;
    double dy = y1 - y0;
    double vx0 = dx - (g0x + dvx0) / 2;
    double vy0 = dy - (g0y + dvy0) / 2;

    double g1x, g1y;
    gravity2(x1, y1, mxa[turn], mya[turn], &g1x, &g1y);
    vx1 = vx = vx0 + dvx0 + (g1x + g0x) / 2;
    vy1 = vy = vy0 + dvy0 + (g1y + g0y) / 2;
}

void Simulator4::step(double dvx, double dvy) {
    dvx = -dvx;
    dvy = -dvy;

    turn++;

    double g0x, g0y;
    gravity2(x, y, mxa[turn], mya[turn], &g0x, &g0y);

    x += vx + (g0x + dvx) / 2;
    y += vy + (g0y + dvy) / 2;

    //turn++;

    double g1x, g1y;
    gravity2(x, y, mxa[turn], mya[turn], &g1x, &g1y);

    vx += dvx + (g0x + g1x) / 2;
    vy += dvy + (g0y + g1y) / 2;
}

void Simulator4::future(int n, vector<double>* xa, vector<double>* ya) {
    xa->reserve(n);
    ya->reserve(n);
    xa->push_back(x);
    ya->push_back(y);
    for (int i = 0; i < n; i++) {
        step();
        xa->push_back(x);
        ya->push_back(y);
    }
}

void Simulator4::future(int n, vector<double>* xa, vector<double>* ya,
                       vector<double>* vxa, vector<double>* vya) {
    xa->reserve(n);
    ya->reserve(n);
    vxa->reserve(n);
    vya->reserve(n);
    xa->push_back(x);
    ya->push_back(y);
    vxa->push_back(vx);
    vya->push_back(vy);
    for (int i = 0; i < n; i++) {
        step();
        xa->push_back(x);
        ya->push_back(y);
        vxa->push_back(vx);
        vya->push_back(vy);
    }
}

void Simulator4::save() {
    x1 = x;
    y1 = y;
    vx1 = vx;
    vy1 = vy;
    turn1 = turn;
}

void Simulator4::restore() {
    x = x1;
    y = y1;
    vx = vx1;
    vy = vy1;
    turn = turn1;
}
